# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ...model import implements, Model
from ...plugins import IPlugin, Service
from ...ui import IDisplay

from .. import IDistributionManager


@implements(IPlugin, IDisplay)
class IDistributionManagerServicePlugin(Model):
    """ The IDistributionManagerServicePlugin class is the plugin definition
    for the default IDistributionManager service.
    """

    # The identifier of the plugin.
    id = 'dip.builder.services.IDistributionManager'

    # The name of the plugin.
    name = "DIP builder distribution manager service plugin"

    def configure(self, plugin_manager):
        """ Configure the plugin. """

        # Create the distribution manager instance.
        from ..default_distribution_manager import DistributionManager
        service = DistributionManager()

        # Bind to the distributions extension point
        pluginmanager.bind_extension_point('dip.builder.distributions',
                IDistributionManager(service), 'distributions')

        # Publish the service.
        pluginmanager.services.append(
                Service(interface=IDistributionManager,
                        implementation=service))
