# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .collection_type_factory import CollectionTypeFactory


class MutableTypeFactory(CollectionTypeFactory):
    """ The MutableTypeFactory class is a base class for all mutable types. """

    def bind(self, model, value, rebind=False):
        """ This is called when a model attribute is being bound or rebound.

        :param model:
            is the model.
        :param value:
            is the attribute's new value.
        :param rebind:
            is set if the attribute already has a value.
        """

        # Mutable types know the model they are contained by.

        if rebind:
            old_value = getattr(model, self.name)
            old_value._dip_model = None

        value._dip_model = model
