# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..toolkits import Toolkit


def Action(parent=None, **args):
    """ Create a toolkit specific action that implements the
    :class:`~dip.ui.IAction` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the action.
    """

    return _factory(Toolkit.action, args, parent)


def CheckBox(parent=None, **args):
    """ Create a toolkit specific check box that implements the
    :class:`~dip.ui.ICheckBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the check box.
    """

    return _factory(Toolkit.check_box, args, parent)


def ComboBox(parent=None, **args):
    """ Create a toolkit specific combo box that implements the
    :class:`~dip.ui.IComboBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the combo box.
    """

    return _factory(Toolkit.combo_box, args, parent)


def Dialog(parent=None, **args):
    """ Create a toolkit specific dialog that implements the
    :class:`~dip.ui.IDialog` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the dialog.
    """

    return _factory(Toolkit.dialog, args, parent)


def Dock(parent=None, **args):
    """ Create a toolkit specific dock that implements the
    :class:`~dip.ui.IDock` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the dock.
    """

    return _factory(Toolkit.dock, args, parent)


def FilesystemLocationEditor(parent=None, **args):
    """ Create a toolkit specific file selector that implements the
    :class:`~dip.ui.IFilesystemLocationEditor` interface.  Note that
    :class:`~dip.pui.StorageLocationEditor` should normally be used instead.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the filesystem location editor.
    """

    return _factory(Toolkit.filesystem_location_editor, args, parent)


def FloatSpinBox(parent=None, **args):
    """ Create a toolkit specific float spin box that implements the
    :class:`~dip.ui.IFloatSpinBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the float spin box.
    """

    return _factory(Toolkit.float_spin_box, args, parent)


def Form(parent=None, **args):
    """ Create a toolkit specific form that implements the
    :class:`~dip.ui.IForm` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the form.
    """

    return _factory(Toolkit.form, args, parent)


def Grid(parent=None, **args):
    """ Create a toolkit specific grid container that implements the
    :class:`~dip.ui.IGrid` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the grid container.
    """

    return _factory(Toolkit.grid, args, parent)


def GroupBox(parent=None, **args):
    """ Create a toolkit specific group box that implements the
    :class:`~dip.ui.IGroupBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the group box.
    """

    return _factory(Toolkit.group_box, args, parent)


def HBox(parent=None, **args):
    """ Create a toolkit specific horizontal box container that implements the
    the :class:`~dip.ui.IHBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the horizontal box container.
    """

    return _factory(Toolkit.h_box, args, parent)


def Label(parent=None, **args):
    """ Create a toolkit specific label that implements the
    :class:`~dip.ui.ILabel` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the label.
    """

    return _factory(Toolkit.label, args, parent)


def LineEditor(parent=None, **args):
    """ Create a toolkit specific line editor that implements the
    :class:`~dip.ui.ILineEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the line editor.
    """

    return _factory(Toolkit.line_editor, args, parent)


def ListEditor(parent=None, **args):
    """ Create a toolkit specific list editor that implements the
    :class:`~dip.ui.IListEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the list editor.
    """

    return _factory(Toolkit.list_editor, args, parent)


def MainWindow(parent=None, **args):
    """ Create a toolkit specific main window that implements the
    :class:`~dip.ui.IMainWindow` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the main window.
    """

    return _factory(Toolkit.main_window, args, parent)


def Menu(parent=None, **args):
    """ Create a toolkit specific menu that implements the
    :class:`~dip.ui.IMenu` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the menu.
    """

    return _factory(Toolkit.menu, args, parent)


def MenuBar(parent=None, **args):
    """ Create a toolkit specific menu bar that implements the
    :class:`~dip.ui.IMenuBar` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the menu bar.
    """

    return _factory(Toolkit.menu_bar, args, parent)


def MessageArea(parent=None, **args):
    """ Create a toolkit specific message area that implements the
    :class:`~dip.ui.IMessageArea` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the message area.
    """

    return _factory(Toolkit.message_area, args, parent)


def OptionList(parent=None, **args):
    """ Create a toolkit specific option list that implements the
    :class:`~dip.ui.IOptionList` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the option list.
    """

    return _factory(Toolkit.option_list, args, parent)


def PushButton(parent=None, **args):
    """ Create a toolkit specific push button that implements the
    :class:`~dip.ui.IPushButton` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the push button.
    """

    return _factory(Toolkit.push_button, args, parent)


def RadioButtons(parent=None, **args):
    """ Create a toolkit specific set of radio buttons that implements the
    :class:`~dip.ui.IRadioButtons` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the radio buttons.
    """

    return _factory(Toolkit.radio_buttons, args, parent)


def SpinBox(parent=None, **args):
    """ Create a toolkit specific int spin box that implements the
    :class:`~dip.ui.ISpinBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the int spin box.
    """

    return _factory(Toolkit.spin_box, args, parent)


def Splitter(parent=None, **args):
    """ Create a toolkit specific splitter that implements the
    :class:`~dip.ui.ISplitter` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the splitter.
    """

    return _factory(Toolkit.splitter, args, parent)


def StorageLocationEditor(parent=None, **args):
    """ Create a toolkit specific file selector that implements the
    :class:`~dip.ui.IStorageLocationEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the storage location editor.
    """

    return _factory(Toolkit.storage_location_editor, args, parent)


def TabBar(parent=None, **args):
    """ Create a toolkit specific tab bar that implements the
    :class:`~dip.ui.ITabBar` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the tab bar.
    """

    return _factory(Toolkit.tab_bar, args, parent)


def TableEditor(parent=None, **args):
    """ Create a toolkit specific table editor that implements the
    :class:`~dip.ui.ITableEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the table editor.
    """

    return _factory(Toolkit.table_editor, args, parent)


def TabPage(parent=None, **args):
    """ Create a toolkit specific tab page that implements the
    :class:`~dip.ui.ITabPage` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the tab page.
    """

    return _factory(Toolkit.tab_page, args, parent)


def TextEditor(parent=None, **args):
    """ Create a toolkit specific text editor that implements the
    :class:`~dip.ui.ITextEditor` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the text editor.
    """

    return _factory(Toolkit.text_editor, args, parent)


def ToolButton(parent=None, **args):
    """ Create a toolkit specific tool button that implements the
    :class:`~dip.ui.IToolButton` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the tool button.
    """

    return _factory(Toolkit.tool_button, args, parent)


def VBox(parent=None, **args):
    """ Create a toolkit specific vertical box container that implements the
    :class:`~dip.ui.IVBox` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the vertical box container.
    """

    return _factory(Toolkit.v_box, args, parent)


def ViewStack(parent=None, **args):
    """ Create a toolkit specific view stack that implements the
    :class:`~dip.ui.IViewStack` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the view stack.
    """

    return _factory(Toolkit.view_stack, args, parent)


def Wizard(parent=None, **args):
    """ Create a toolkit specific wizard that implements the
    :class:`~dip.ui.IWizard` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the wizard.
    """

    return _factory(Toolkit.wizard, args, parent)


def WizardPage(parent=None, **args):
    """ Create a toolkit specific wizard page that implements the
    :class:`~dip.ui.IWizardPage` interface.

    :param parent:
        is the optional parent view.
    :param \*\*args:
        are the initial attribute and property values.
    :return:
        the wizard page.
    """

    return _factory(Toolkit.wizard_page, args, parent)


def _factory(factory, args, parent):
    """ Use the default toolkit to create a configured object. """

    obj = factory(parent)

    if args:
        # Go through any arguments setting those that correspond to interface
        # attributes and saving the rest which must therefore be toolkit view
        # specific properties.
        properties = {}

        for aname, avalue in args.items():
            if hasattr(obj, aname):
                setattr(obj, aname, avalue)
            else:
                properties[aname] = avalue

        obj.configure(properties)

    return obj
