# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ....model import adapt, observe, unadapted
from ....pui import Dock, TabPage
from ....ui import Application, IDisplay, IMainWindow

from ...i_action_hints import IActionHints
from ...i_area_hints import IAreaHints
from ...base_shell_adapter import BaseShellAdapter
from ...i_shell import IShell


@adapt(IMainWindow, to=IShell)
class IMainWindowIShellAdapter(BaseShellAdapter):
    """ Adapt IMainWindow to IShell. """

    @observe('views')
    def __views_changed(self, change):
        """ Invoked when the list of views changes. """

        main_window = IMainWindow(self.adaptee)
        tab_bar = main_window.view

        # Remove any old views.
        for view in change.old:
            area_hints = IAreaHints(view)

            if area_hints.area == '':
                # Find the tab page.
                for tab_page in tab_bar.views:
                    if tab_page.view is view:
                        tab_bar.views.remove(tab_page)
                        break
            else:
                # Find the dock.
                for dock in main_window.docks:
                    if dock.view is view:
                        main_window.docks.remove(dock)
                        break

        # Add any new views.
        for view in change.new:
            area_hints = IAreaHints(view)
            idisplay = IDisplay(view)

            if area_hints.area in '':
                tab_page = TabPage()
                tab_page.view = view

                tab_bar.views.append(tab_page)

                self._configure_tab_page(idisplay)
                observe('name', idisplay,
                        lambda c: self._configure_tab_page(c.model))

                tab_bar.current_page = tab_page
                Application().active_view = view
            else:
                dock = Dock()
                dock.view = view
                dock.window_title = idisplay.name

                # Convert the shell area identifier to a main window dock area.
                dock_area = area_hints.area.split('.')[-1]
                dock.area = dock_area

                action_hints = IActionHints(view)
                dock.id = action_hints.id
                dock.within = action_hints.within

                main_window.docks.append(dock)

        # Update the visibility of the tabs.
        nr_views = len(tab_bar.views)
        tab_bar.tabs_visible = (nr_views > 1 or (nr_views == 1 and self.main_area_policy == 'multi'))

    def _configure_tab_page(self, idisplay):
        """ Set the title and tooltip of a tab page. """

        # Find the tab page.
        for tab_page in IMainWindow(self.adaptee).view.views:
            if IDisplay(tab_page.view) is idisplay:
                # Use the short name if there is one.
                title = idisplay.short_name
                tool_tip = idisplay.name

                if title == "":
                    title = tool_tip
                    tool_tip = ""

                tab_page.title = title
                tab_page.tool_tip = tool_tip

                break
