# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QAction

from ....model import adapt, notify_observers
from ....ui import IAction

from ..object_adapter import ObjectAdapter


@adapt(QAction, to=IAction)
class QActionIActionAdapter(ObjectAdapter):
    """ An adapter to implement IAction for a QAction. """

    def configure(self, properties):
        """ Configure the action. """

        self.adaptee.triggered[bool].connect(self._on_triggered)

        super().configure(properties)

    @IAction.checkable.getter
    def checkable(self):
        """ The getter for the action's checkable state. """

        return self.adaptee.isCheckable()

    @checkable.setter
    def checkable(self, value):
        """ The setter for the action's checkable state. """

        self.adaptee.setCheckable(value)

    @IAction.checked.getter
    def checked(self):
        """ The getter for the action's checked state. """

        return self.adaptee.isChecked()

    @checked.setter
    def checked(self, value):
        """ The setter for the action's checked state. """

        self.adaptee.setChecked(value)

    @IAction.enabled.getter
    def enabled(self):
        """ The getter for the action's enabled state. """

        return self.adaptee.isEnabled()

    @enabled.setter
    def enabled(self, value):
        """ The setter for the action's enabled state. """

        self.adaptee.setEnabled(value)

    @IAction.plain_text.getter
    def plain_text(self):
        """ The getter for the action's plain text. """

        return self.adaptee.text().replace('&', '').replace('...', '')

    @IAction.shortcut.getter
    def shortcut(self):
        """ The getter for the action's shortcut. """

        return self.adaptee.shortcut().toString()

    @shortcut.setter
    def shortcut(self, value):
        """ The setter for the action's shortcut. """

        self.adaptee.setShortcut(value)

    @IAction.status_tip.getter
    def status_tip(self):
        """ The getter for the action's status tip. """

        return self.adaptee.statusTip()

    @status_tip.setter
    def status_tip(self, value):
        """ The setter for the action's status tip. """

        self.adaptee.setStatusTip(value)

    @IAction.text.getter
    def text(self):
        """ The getter for the action's text. """

        return self.adaptee.text()

    @text.setter
    def text(self, value):
        """ The setter for the action's text. """

        self.adaptee.setText(value)

    @IAction.tool_tip.getter
    def tool_tip(self):
        """ The getter for the action's tool tip. """

        return self.adaptee.toolTip()

    @tool_tip.setter
    def tool_tip(self, value):
        """ The setter for the action's tool tip. """

        self.adaptee.setToolTip(value)

    @IAction.visible.getter
    def visible(self):
        """ The getter for the action's visible state. """

        return self.adaptee.isVisible()

    @visible.setter
    def visible(self, value):
        """ The setter for the action's visible state. """

        self.adaptee.setVisible(value)

    @IAction.whats_this.getter
    def whats_this(self):
        """ The getter for the action's "Whats This?" help. """

        return self.adaptee.whatsThis()

    @whats_this.setter
    def whats_this(self, value):
        """ The setter for the action's "Whats This?" help. """

        self.adaptee.setWhatsThis(value)

    def _on_triggered(self, checked):
        """ Invoked when the action is triggered. """

        if self.when_triggered is not None:
            self.when_triggered(self)

        self.trigger = True

        if self.adaptee.isCheckable():
            notify_observers('checked', self, checked, not checked)
