# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import QLineEdit
from PyQt4.QtTest import QTest

from ....automate import IAutomatedEditor
from ....model import adapt, Adapter
from ....ui import ILineEditor

from ..editor_widget_adapter import EditorWidgetAdapter


@adapt(QLineEdit, to=ILineEditor)
class QLineEditILineEditorAdapter(EditorWidgetAdapter):
    """ An adapter to implement ILineEditor for a QLineEdit. """

    def configure(self, properties):
        """ Configure the editor. """

        self.adaptee.textEdited.connect(self.qt4_notify)

        super().configure(properties)

    @ILineEditor.read_only.getter
    def read_only(self):
        """ Invoked to get the read-only status. """

        return self.adaptee.isReadOnly()

    @read_only.setter
    def read_only(self, value):
        """ Invoked to set the read-only status. """

        self.adaptee.setReadOnly(value)

    @ILineEditor.value.getter
    def value(self):
        """ Invoked to get the editor's value. """

        return self.adaptee.text()

    @value.setter
    def value(self, value):
        """ Invoked to set the editor's value. """

        # Save for the notifier.
        self.qt4_old_value = value

        self.adaptee.setText(value)


@adapt(QLineEdit, to=IAutomatedEditor)
class QLineEditIAutomatedEditorAdapter(Adapter):
    """ An adapter to implement IAutomatedEditor for the widget. """

    def simulate_set(self, id, delay, value):
        """ Simulate the user setting the editor value.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param value:
            is the value to set.
        """

        widget = self.adaptee

        QTest.keyClicks(widget, 'a', Qt.ControlModifier, delay)
        QTest.keyClicks(widget, value, Qt.NoModifier, delay)
