# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import QPoint, Qt
from PyQt4.QtGui import QPushButton
from PyQt4.QtTest import QTest

from ....automate import IAutomatedTrigger
from ....model import adapt, Adapter, notify_observers
from ....ui import IPushButton

from ..editor_widget_adapter import EditorWidgetAdapter


@adapt(QPushButton, to=IPushButton)
class QPushButtonIPushButtonAdapter(EditorWidgetAdapter):
    """ An adapter to implement IPushButton for a QPushButton. """

    # The label is embedded.
    label_policy = 'embedded'

    def configure(self, properties):
        """ Configure the editor. """

        self.adaptee.clicked.connect(self._on_clicked)

        super().configure(properties)

    @IPushButton.label.getter
    def label(self):
        """ Invoke to get the label. """

        return self.adaptee.text()

    @label.setter
    def label(self, value):
        """ Invoked to set the label. """

        self.adaptee.setText(value)

    @IPushButton.value.getter
    def value(self):
        """ Invoke to get the editor's value. """

        # Any value will do as it is only used to pull the Trigger.
        return True

    @value.setter
    def value(self, value):
        """ Invoked to set the editor's value. """

        raise ValueError("cannot set the value of a QPushButton")

    def _on_clicked(self):
        """ Invoked when clicked. """

        notify_observers('value', self, True, False)


@adapt(QPushButton, to=IAutomatedTrigger)
class QPushButtonIAutomatedTriggerAdapter(Adapter):
    """ An adapter to implement IAutomatedTrigger for the widget. """

    def simulate_trigger(self, id, delay):
        """ Simulate the user pulling the trigger.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        """

        QTest.mouseClick(self.adaptee, Qt.LeftButton, Qt.NoModifier, QPoint(),
                delay)
