# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QStackedLayout

from ....model import adapt, notify_observers, observe
from ....ui import IViewStack

from ..layout_adapter import LayoutAdapter
from ..utils import as_QWidget, from_QWidget


@adapt(QStackedLayout, to=IViewStack)
class QStackedLayoutIViewStackAdapter(LayoutAdapter):
    """ An adapter to implement IViewStack for a QStackedLayout. """

    def configure(self, properties):
        """ Configure the view. """

        self._qt4_old_view = None

        self.adaptee.currentChanged.connect(self._current_changed)

        super().configure(properties)

    @IViewStack.current_view.getter
    def current_view(self):
        """ Invoked to get the current view. """

        return from_QWidget(self.adaptee.currentWidget())

    @current_view.setter
    def current_view(self, value):
        """ Invoked to set the current view. """

        self.adaptee.setCurrentWidget(as_QWidget(value))

    @observe('views')
    def __views_changed(self, change):
        """ Invoked when the list of views changes. """

        layout = self.adaptee

        # Remove any old views.
        for view in change.old:
            qwidget = as_QWidget(view)
            layout.removeWidget(qwidget)

            # For potential garbage collection.
            qwidget.setParent(None)

        # Add any new views.
        for view in change.new:
            layout.addWidget(as_QWidget(view))

    def _current_changed(self, idx):
        """ Invoked when the current view changes. """

        new_view = from_QWidget(self.adaptee.widget(idx))

        old_view = self._qt4_old_view
        self._qt4_old_view = new_view

        notify_observers('current_view', self, new_view, old_view)
