# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QLayout, QVBoxLayout, QWidget

from ...model import unadapted
from ...ui import IView


def as_QLayout(view):
    """ Return a view as a QLayout. """

    view = unadapted(view)

    if isinstance(view, QWidget):
        layout = _QWidgetWrapper()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(view)

        view = layout

    return view


def from_QLayout(view):
    """ Return a view from a QLayout. """

    if view is None:
        return None

    if isinstance(view, _QWidgetWrapper):
        view = view.itemAt(0).widget()

    return IView(view)


class _QWidgetWrapper(QVBoxLayout):
    """ A wrapper for a QWidget when a QLayout is needed. """


def as_QWidget(view):
    """ Return a view as a QWidget. """

    view = unadapted(view)

    if isinstance(view, QLayout):
        # See if it already has a wrapper.
        widget = view.parentWidget()

        if widget is None:
            widget = _QLayoutWrapper()
            widget.setLayout(view)

            # Set the focus proxy if there is one.
            focus_proxy = getattr(view, 'qt4_focus_proxy', None)
            if focus_proxy is not None:
                widget.setFocusProxy(focus_proxy)

            widget.setWindowTitle(IView(view).window_title)

        view = widget

    return view


def from_QWidget(view):
    """ Return a view from a QWidget. """

    if view is None:
        return None

    if isinstance(view, _QLayoutWrapper):
        view = view.layout()

    return IView(view)


class _QLayoutWrapper(QWidget):
    """ A wrapper for a QLayout when a QWidget is needed. """

    def closeEvent(self, event):
        """ Invoked when the widget wants to close. """

        view = self.layout()

        for handler in IView(view).close_request_handlers:
            if not handler(view):
                event.ignore()
                break
        else:
            event.accept()


def as_QWidget_parent(view):
    """ Return a view as a QWidget in the object hierachy. """

    view = unadapted(view)

    while view is not None and not isinstance(view, QWidget):
        view = view.parent()

    return view
