# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import pyqtSignal
from PyQt4.QtGui import QToolBar


class DipAddRemoveToolBar(QToolBar):
    """ The DipAddRemoveToolBar class implements a tool bar widget that
    contains buttons to add and remove items from a collection.
    """

    # This is emitted when the add button is triggered.
    addTriggered = pyqtSignal()

    # This is emitted when the remove button is triggered.
    removeTriggered = pyqtSignal()

    def __init__(self, parent=None):
        """ Initialise the widget.

        :param parent:
            is the optional parent widget.
        """

        super().__init__(parent)

        action = self.addAction("+")
        action.triggered[()].connect(self.addTriggered)
        action.setToolTip("Add")
        self.widgetForAction(action).setObjectName('add')

        self._remove_action = action = self.addAction("-")
        action.triggered[()].connect(self.removeTriggered)
        action.setToolTip("Remove")
        self.widgetForAction(action).setObjectName('remove')

    def setRemoveEnabled(self, enabled):
        """ Set the enabled state of the remove button.

        :param enabled:
            is ``True`` if the button is enabled.
        """

        self._remove_action.setEnabled(enabled)
