# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .editor_factory import EditorFactory


class FloatSpinBox(EditorFactory):
    """ The FloatSpinBox class implements a spin box editor factory for float
    attributes.
    """

    # The name of the toolkit factory method.
    toolkit_factory = 'float_spin_box'

    def configure_with_attribute_type(self, view, attribute_type):
        """ Configure the view possibly using values from the bound attribute's
        type.

        :param view:
            is the view to configure.
        :param attribute_type:
            is the attribute's type.
        """

        metadata = attribute_type.metadata

        maximum = metadata.get('maximum')
        if maximum is not None:
            view.maximum = maximum

        minimum = metadata.get('minimum')
        if minimum is not None:
            view.minimum = minimum

        super().configure_with_attribute_type(view, attribute_type)
