# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Bool, Enum, Trigger, Tuple

from .i_single_view_container import ISingleViewContainer


# The different dialog buttons.
DialogButton = Enum('ok', 'cancel', 'yes', 'no')


class IDialog(ISingleViewContainer):
    """ The IDialog interface defines the API to be implemented by a dialog.
    """

    # This is set, typically by a :term:`controller`, when the contents of the
    # dialog are acceptable.
    acceptable = Bool(True)

    # This is pulled when the user accepts the dialog.
    accepted = Trigger()

    # The dialog buttons.
    buttons = Tuple(DialogButton, default=('ok', 'cancel'))

    def execute(self):
        """ Execute the dialog as a modal dialog.

        :return:
            ``True`` if the user accepted the dialog.
        """
