# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import adapt, Adapter, Instance, Model, Str, TypeFactory

from .i_editor import IEditor
from .i_view_binding import IViewBinding


class IEditorBinding(IViewBinding):
    """ The IEditorBinding interface defines additional data to be bound to an
    editor.
    """

    # The name of the attribute in the model.  This is not an
    # :term:`attribute path`.
    attribute_name = Str()

    # The type of the attribute in the model.
    attribute_type = Instance(TypeFactory)

    # The model containing the attribute.
    model = Instance(Model)


@adapt(IEditor, to=IEditorBinding)
class IEditorIEditorBindingAdapter(Adapter):
    """ The IEditorIEditorBindingAdapter adapts IEditor to IEditorBinding. """
