# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Bool, Instance, List, Str

from .i_action import IAction
from .i_container import IContainer


class IMenu(IContainer):
    """ The IMenu interface defines the API to be implemented by a menu. """

    # The menu's title.
    title = Str()

    # The visibility of the menu.  If this is None then the menu will be
    # visible only when it has some content.
    visible = Bool(None, allow_none=True)

    # The menu's contents.  An empty string means a separator.
    views = List(Instance(IAction, 'dip.ui.IMenu', Str()))

    def add_action(self, action):
        """ Try and add an action or action collection to the menu.

        :param action:
            is the action.
        :return:
            ``True`` if the action was added.  ``False`` is returned if the
            menu couldn't accommodate the action's desired positioning.
        """
