# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Enum

from .i_editor_binding import IEditorBinding
from .editor_factory import EditorFactory


class LineEditor(EditorFactory):
    """ The LineEditor class implements a line editor factory for string
    attributes.
    """

    # This specifies whether or not a value is required.  It is ignored if a
    # validator is explicitly set.  'no' means that a value is not required
    # (i.e. it can be an empty string).  'yes' means that a value is required.
    # 'stripped' means that a value with no leading or trailing whitespace is
    # required.  If it is not explicitly set then any corresponding value in
    # the type's meta-data is used.
    required = Enum('no', 'yes', 'stripped', allow_none=True)

    # The name of the toolkit factory method.
    toolkit_factory = 'line_editor'

    def set_default_validator(self, view):
        """ Sets a view's default validator.

        :param view:
            is the view.
        """

        # See if a value is required.
        required = self.required
        if required is None:
            attribute_type = IEditorBinding(view).attribute_type
            if attribute_type is not None:
                required = attribute_type.metadata.get('required')

            if required is None:
                required = 'no'

        # See if a validator is required.
        if required in ('yes', 'stripped'):
            from .string_validator import StringValidator

            view.validator = StringValidator(stripped=(required == 'stripped'))
