# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Bool, implements, Model

from .i_validator import IValidator


@implements(IValidator)
class StringValidator(Model):
    """ The StringValidator class implements an editor validator for required
    string values.
    """

    # This specifies that the value should not have leading or trailing spaces.
    stripped = Bool(False)

    def configure(self, editor):
        """ Configure an editor to use this validator.

        :param editor:
            is the editor.
        """

    def validate(self, controller, editor, value):
        """ Validate an editor's value.

        :param controller:
            is the controller.
        :param editor:
            is the editor.
        :param value:
            is the value.
        :return:
            a string describing why the value is invalid.  An empty string is
            returned if the value is valid.
        """

        if self.stripped:
            if value == '' or value != value.strip():
                return "a value without leading or trailing spaces is required"
        elif value == '':
            return "a value is required"

        return ''
