# Copyright (c) 2009 Riverbank Computing Limited.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from dip.model import Enum, Model, ValidationError


class TestEnumType(unittest.TestCase):
    """ This tests the Enum type. """

    def test_invalid_enum_type(self):
        """ Invalid Enum type. """

        with self.assertRaises(ValidationError):
            class Klass(Model):
                attr = Enum()

    def test_invalid_member_enum_type(self):
        """ Invalid member of an Enum type. """

        with self.assertRaises(ValidationError):
            class Klass(Model):
                attr = Enum(False)

    def test_valid_enum_type(self):
        """ Valid Enum type. """

        class Klass(Model):
            attr = Enum('Yes', 'No', 'Maybe')

        k = Klass()
        self.assertEqual(k.attr, 'Yes')

        k = Klass(attr='No')
        self.assertEqual(k.attr, 'No')

        with self.assertRaises(ValidationError):
            Klass(attr=False)

    def test_explicit_default_enum_type(self):
        """ Enum type with an explicit default. """

        class Klass(Model):
            attr = Enum('Yes', 'No', 'Maybe', default='Maybe')

        k = Klass()
        self.assertEqual(k.attr, 'Maybe')


if __name__ == '__main__':
    unittest.main()
