# Copyright (c) 2012 Riverbank Computing Limited.


if __name__ == '__main__':
    import sys
    from PyQt4.QtGui import QApplication

    app = QApplication(sys.argv)


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtGui import QLineEdit

from dip.automate import Robot
from dip.model import Model, Str
from dip.toolkits import Toolkit
from dip.ui import Bindings, LineEditor


class TestBinding(unittest.TestCase):
    """ This tests binding to explicitly created GUIs. """

    def setUp(self):
        """ Make sure the toolkit is referenced in order to import adapters.
        """

        Toolkit().instance

    def test_str_model(self):
        """ Test bound string attributes of a model. """

        class TestModel(Model):
            attr = Str('default')

        model = TestModel()
        ui = QLineEdit(objectName='BoundModel_str')
        bindings = Bindings(BoundModel_str='attr')
        bindings.bind(ui, model=model)
        ui.show()

        Robot.simulate('BoundModel_str', 'set', 'non-default')
        self.assertEqual(model.attr, 'non-default')

    def test_str_mapping(self):
        """ Test bound string attributes of a mapping. """

        model = dict(attr='default')
        ui = QLineEdit(objectName='BoundMapping_str')
        bindings = Bindings(BoundMapping_str='attr')
        bindings.bind(ui, model=model)
        ui.show()

        Robot.simulate('BoundMapping_str', 'set', 'non-default')
        self.assertEqual(model['attr'], 'non-default')

    def test_valid_factory(self):
        """ Test binding a valid factory. """

        model = dict(attr='default')
        ui = QLineEdit(objectName='BoundValidFactory')
        bindings = Bindings(BoundValidFactory=LineEditor('attr'))
        bindings.bind(ui, model=model)
        ui.show()

        Robot.simulate('BoundValidFactory', 'set', 'non-default')
        self.assertEqual(model['attr'], 'non-default')

    def test_invalid_factory(self):
        """ Test binding an invalid factory. """

        model = dict(attr='default')
        ui = QLineEdit(objectName='BoundInvalidFactory')
        bindings = Bindings(BoundInvalidFactory=list())

        with self.assertRaises(TypeError):
            bindings.bind(ui, model=model)

    def test_invalid_editor(self):
        """ Test binding an invalid editor. """

        model = dict(attr='default')
        ui = QLineEdit(objectName='BoundInvalidEditor')
        bindings = Bindings(NoSuchEditor='attr')

        with self.assertRaises(NameError):
            bindings.bind(ui, model=model)

    def test_invalid_attribute(self):
        """ Test binding an invalid attribute. """

        model = dict(attr='default')
        ui = QLineEdit(objectName='BoundInvalidAttribute')
        bindings = Bindings(BoundInvalidAttribute='no_such_attr')

        with self.assertRaises(AttributeError):
            bindings.bind(ui, model=model)


if __name__ == '__main__':
    unittest.main()
