# Copyright (c) 2012 Riverbank Computing Limited.


if __name__ == '__main__':
    import sys
    from PyQt4.QtGui import QApplication

    app = QApplication(sys.argv)


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from dip.automate import Robot
from dip.ui import Dialog


class TestDialog(unittest.TestCase):
    """ This tests the Dialog view. """

    def test_ok(self):
        """ Test Dialog Ok button. """

        model = dict(attr=10)
        view_factory = Dialog(id='Dialog.ok')
        view = view_factory(model=model)

        robot = Robot()
        robot.record('Dialog.ok:attr', 'set', 20)
        robot.record('Dialog.ok', 'click', 'ok')
        robot.play(after=0)

        accepted = view.execute()

        # Verify that the dialog returned the expected value and that the
        # model was updated.
        self.assertTrue(accepted)
        self.assertEqual(model['attr'], 20)

    def test_cancel(self):
        """ Test Dialog Cancel button. """

        model = dict(attr=10)
        view_factory = Dialog(id='Dialog.cancel')
        view = view_factory(model=model)

        robot = Robot()
        robot.record('Dialog.cancel:attr', 'set', 20)
        robot.record('Dialog.cancel', 'click', 'cancel')
        robot.play(after=0)

        accepted = view.execute()

        # Verify that the dialog returned the expected value and that the
        # model was not updated.
        self.assertFalse(accepted)
        self.assertEqual(model['attr'], 10)


if __name__ == '__main__':
    unittest.main()
