# Copyright (c) 2012 Riverbank Computing Limited.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtGui import QApplication
from PyQt4.QtTest import QTest

from dip.automate import AutomationError, Robot
from dip.model import List, Model, Str, unadapted
from dip.ui import ListEditor, VBox


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)


# The test model.
class TestModel(Model):
    attr = List(Str())

model = None


# The test GUI.
view_factory = VBox(ListEditor('attr'), id='ListEditorType')

view = None


class TestListEditorWithType(unittest.TestCase):
    """ This tests the list editor when an element is a simple type. """

    def setUp(self):
        """ Initialise the list widget, creating it if necessary. """

        # Create it if this is the first call.
        global model, view

        if model is None:
            model = TestModel()

        if view is None:
            view = view_factory(model=model)

        model.attr = []

        view.visible = True
        QTest.qWaitForWindowShown(unadapted(view))
        QApplication.setActiveWindow(unadapted(view))

    def tearDown(self):
        """ Tidy up the list widget. """

        view.visible = False

    def test_update(self):
        """ Test update. """

        model.attr = ["Old"]
        Robot.simulate('ListEditorType:attr', 'update', "New", 0)
        self.assertEqual(model.attr, ["New"])

    def test_update_invalid_index(self):
        """ Test update with an invalid index. """

        with self.assertRaises(AutomationError):
            Robot.simulate('ListEditorType:attr', 'update', "Invalid", -1)

    def test_append(self):
        """ Test append. """

        Robot.simulate('ListEditorType:attr', 'append', "New")
        self.assertEqual(model.attr, ["New"])

    def test_remove(self):
        """ Test remove. """

        model.attr = ["First", "Second"]
        Robot.simulate('ListEditorType:attr', 'remove', 0)
        self.assertEqual(model.attr, ["Second"])

    def test_move(self):
        """ Test move. """

        model.attr = ["First", "Second", "Third"]
        Robot.simulate('ListEditorType:attr', 'move', 0, 2)
        self.assertEqual(model.attr, ["Second", "Third", "First"])


if __name__ == '__main__':
    unittest.main()
