# Copyright (c) 2012 Riverbank Computing Limited.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtGui import QApplication
from PyQt4.QtTest import QTest

from dip.automate import Robot
from dip.model import unadapted
from dip.ui import Form


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)


class TestMapping(unittest.TestCase):
    """ This tests views based on a Python mapping type. """

    def test_bool(self):
        """ Test bool attributes of a mapping. """

        model = dict(attr=False)
        view_factory = Form(id='Mapping.bool')
        view = view_factory(model=model)
        view.visible = True

        Robot.simulate('Mapping.bool:attr', 'set', True)
        self.assertIs(model['attr'], True)

    def test_int(self):
        """ Test integer attributes of a mapping. """

        model = dict(attr=10)
        view_factory = Form(id='Mapping.int')
        view = view_factory(model=model)
        view.visible = True

        Robot.simulate('Mapping.int:attr', 'set', 20)
        self.assertEqual(model['attr'], 20)

    def test_float(self):
        """ Test float attributes of a mapping. """

        model = dict(attr=10.0)
        view_factory = Form(id='Mapping.float')
        view = view_factory(model=model)
        view.visible = True

        Robot.simulate('Mapping.float:attr', 'set', 20)
        self.assertEqual(model['attr'], 20.0)

    def test_str(self):
        """ Test string attributes of a mapping. """

        model = dict(attr='default')
        view_factory = Form(id='Mapping.str')
        view = view_factory(model=model)
        view.visible = True

        Robot.simulate('Mapping.str:attr', 'set', 'non-default')
        self.assertEqual(model['attr'], 'non-default')

    def test_list(self):
        """ Test list attributes of a mapping. """

        model = dict(attr=['first', 'second'])
        view_factory = Form(id='Mapping.list')
        view = view_factory(model=model)
        view.visible = True
        QTest.qWaitForWindowShown(unadapted(view))
        QApplication.setActiveWindow(unadapted(view))

        Robot.simulate('Mapping.list:attr', 'update', '2nd', 1)
        self.assertEqual(model['attr'], ['first', '2nd'])


if __name__ == '__main__':
    unittest.main()
